(function () {
  window.__frontend = {
    version: '5.8.1',
    deps: {
      '@labelu/audio-annotator-react': '1.8.1',
      '@labelu/components-react': '1.7.9',
      '@labelu/formatter': '1.0.2',
      '@labelu/i18n': '1.0.6',
      '@labelu/image': '1.4.0',
      '@labelu/image-annotator-react': '2.4.1',
      '@labelu/interface': '1.3.1',
      '@labelu/video-annotator-react': '1.4.7',
      '@labelu/video-react': '1.5.1',
    },
  };
})();
